#include <iostream>
#include <fstream>
#include <map>
#include <set>
#include <string>
#include <vector>

#include "util.h"
#include "encoder.h"
#include "model.h"
#include "string_vector.h"

using namespace SD;


int save( const char* modelfile, const Encoder& encoder, const StringVector& labelset, const Model& model ) {

    FILE* os = fopen( modelfile, "wb" );
    if ( os == NULL ) {
	fprintf( stderr, HERE "cannot open %s\n", modelfile );
	return -1;
    }
    if ( encoder.save( os ) == -1 ) return -1;
    if ( labelset.save( os ) == -1 ) return -1;
    if ( model.save( os ) == -1 ) return -1;
    fclose( os );
    return 0;
}


int read( const char* filename, Encoder& encoder, StringVector& labelset, Model& model ) {

    StringVector word;
    StringVector prefix;
    StringVector suffix;
    
    FILE* is = fopen( filename, "rb" );
    if ( is == NULL ) {
	fprintf( stderr, HERE "cannot open %s\n", filename );
	return -1;
    }
    if ( word.read( is ) == -1 ) return -1;
    if ( prefix.read( is ) == -1 ) return -1;
    if ( suffix.read( is ) == -1 ) return -1;
    if ( labelset.read( is ) == -1 ) return -1;
    if ( model.read( is ) == -1 ) return -1;
    fclose( is );

    if ( encoder.build( word.vector(), prefix.vector(), suffix.vector() ) == -1 ) return -1;
    
    return 0;
}

    
int main( int argc, const char** argv ) {

    if ( argc != 3 ) {
	fprintf( stderr, "%s <model (text)> <model (binary)>\n", argv[0] );
	return -1;
    }
    const char* modelfile1 = argv[argc-2];
    const char* modelfile2 = argv[argc-1];
    
    Encoder      encoder;
    StringVector labelset;
    Model        model;
    if ( read( modelfile1, encoder, labelset, model ) == -1 ) return -1;
    if ( save( modelfile2, encoder, labelset, model ) == -1 ) return -1;
    
    return 0;
}
