#ifndef __DenseMatrix__
#define __DenseMatrix__

#include <iostream>
#include <fstream>
#include <algorithm> // fill, copy, max_element
#include "util.h"

namespace SD {
    
template<class T>
class DenseMatrix {
  private:
    uint row_size_;
    uint column_size_;
    T* data_;
  public:
    DenseMatrix();
    DenseMatrix( const DenseMatrix& obj );
    ~DenseMatrix();
    DenseMatrix& operator=( const DenseMatrix& obj );
    int read( FILE* is );
    int load( FILE* is );
    int save( FILE* os ) const;
    void init( uint row_size, uint column_size, T init_val=0.0 );
    uint row_size() const;
    uint column_size() const;
    const T* operator[]( uint row ) const;
    T* operator[]( uint row );
    T max() const;
    T max( uint first_row, uint last_row, uint first_column, uint last_column ) const;
};


template<class T>
DenseMatrix<T>::DenseMatrix() {

    row_size_    = 0;
    column_size_ = 0;
    data_        = NULL;
}


template<class T>
DenseMatrix<T>::~DenseMatrix() {

    delete[] data_;
}


template<class T>
DenseMatrix<T>& DenseMatrix<T>::operator=( const DenseMatrix<T>& obj ) {

    row_size_    = obj.row_size();
    column_size_ = obj.column_size();

    delete[] data_;
    data_ = new T[row_size_*column_size_];
    std::copy( obj[0], obj[0] + row_size_*column_size_, data_ );
    
    return *this;
}

    
template<class T>
int DenseMatrix<T>::read( FILE* is ) {

    uint i = 0;
    char* s = new char[BUFF_SIZE];
    while ( fgets( s, BUFF_SIZE, is ) != NULL ) {
	if ( s[strlen(s)-1] != '\n' ) {
	    fprintf( stderr, HERE "too long input!\n" );
	    continue;
	}
	s[strlen(s)-1] = '\0';

	if ( i == 0 ) {
	    row_size_ = atoi( s );
	}else if ( i == 1 ) {
	    column_size_ = atoi( s );
	    delete[] data_;
	    data_ = new T[row_size_*column_size_];
	}else {
	    data_[i-2] = static_cast<T>( atof( s ) );
	}
	++i;
	if ( i == row_size_*column_size_ + 2 ) break;
    }
    delete[] s;
    
    return 0;
}


template<class T>
int DenseMatrix<T>::load( FILE* is ) {

    if ( fread( &row_size_, sizeof( uint ), 1, is ) != 1 ) {
	fprintf( stderr, HERE "failed to load\n" );
	return -1;
    }
    if ( fread( &column_size_, sizeof( uint ), 1, is ) != 1 ) {
	fprintf( stderr, HERE "failed to load\n" );
	return -1;
    }
    delete[] data_;
    data_ = new T[row_size_*column_size_];
    if ( fread( data_, sizeof( T ), row_size_*column_size_, is ) != row_size_*column_size_ ) {
	fprintf( stderr, HERE "failed to load\n" );
	return -1;
    }
    
    return 0;
}


template<class T>
int DenseMatrix<T>::save( FILE* os ) const {

    if ( fwrite( &row_size_, sizeof( uint ), 1, os ) != 1 ) {
	fprintf( stderr, HERE "failed to save\n" );
	return -1;
    }
    if ( fwrite( &column_size_, sizeof( uint ), 1, os ) != 1 ) {
	fprintf( stderr, HERE "failed to save\n" );
	return -1;
    }
    if ( fwrite( data_, sizeof( T ), row_size_*column_size_, os ) != row_size_*column_size_ ) {
	fprintf( stderr, HERE "failed to save\n" );
	return -1;
    }

    return 0;
}


template<class T>
uint DenseMatrix<T>::row_size() const {

    return row_size_;
}


template<class T>
uint DenseMatrix<T>::column_size() const {

    return column_size_;
}


template<class T>
void DenseMatrix<T>::init( uint row_size, uint column_size, T init_val ) {
    
    row_size_    = row_size;
    column_size_ = column_size;
    delete[] data_;
    data_ = new T[row_size_*column_size_];
    std::fill( data_, data_ + row_size_*column_size_, init_val );
}


template<class T>
inline const T* DenseMatrix<T>::operator[]( uint row ) const {

    return data_ + row*column_size_;
}


template<class T>
inline T* DenseMatrix<T>::operator[]( uint row ) {

    return data_ + row*column_size_;
}


template<class T>
inline T DenseMatrix<T>::max() const {
    
    return max( 0, row_size_, 0, column_size_ );
}


template<class T>
inline T DenseMatrix<T>::max( uint first_row, uint last_row, uint first_column, uint last_column ) const {

    T ret = *(data_ + first_row*column_size_ + first_column);
    for ( uint row = first_row; row < last_row; ++row ) {
	T* it = std::max_element( data_ + row*column_size_ + first_column, data_ + row*column_size_ + last_column );
	if ( ret < *it ) {
	    ret = *it;
	}
    }
    return ret;
}

}

#endif
