#ifndef __ENCODER__
#define __ENCODER__

#include <iostream>
#include <fstream>
#include <vector>

#include "util.h"
#include "token.h"
#include "double_array.h"

namespace SD {

class Encoder {
  private:
    DoubleArray word_trie_;
    DoubleArray prefix_trie_;
    DoubleArray suffix_trie_;
  public:
    Encoder() {}
    Encoder( const Encoder& obj );
    ~Encoder() {}
    Encoder& operator=( const Encoder& obj );
    void run( Token* x, size_t n );
    int build( const std::vector<std::string>& word, const std::vector<std::string>& prefix, const std::vector<std::string>& suffix );
    int load( FILE* is );
    int save( FILE* os ) const;
    size_t word_num() const;
    size_t prefix_num() const;
    size_t suffix_num() const;
};


inline void Encoder::run( Token* x, size_t n ) {

    for ( size_t i = 0; i < n; ++i ) {
	x[i].id = word_trie_.exact_match( x[i].begin, x[i].end );
	
	if ( x[i].id < word_trie_.key_num() ) {
	    x[i].prefix_id      = prefix_trie_.key_num();
	    x[i].suffix_id      = suffix_trie_.key_num();
	    x[i].capitalization = 0;
	}else {
	    x[i].prefix_id      = prefix_trie_.longest_match( x[i].begin, x[i].end );
	    x[i].suffix_id      = suffix_trie_.longest_match_reverse( x[i].begin, x[i].end );
	    x[i].capitalization = 0;
	    for ( const char* s = x[i].begin; s != x[i].end; ++s ) {
		if ( 'A' <= *s && *s <= 'Z' ) {
		    x[i].capitalization = 1;
		    break;
		}
	    }
	}
    }
}


int Encoder::load( FILE* is ) {

    if ( word_trie_.load( is ) == -1 ) return -1;
    if ( prefix_trie_.load( is ) == -1 ) return -1;
    if ( suffix_trie_.load( is ) == -1 ) return -1;
    
    BOS_ID = word_trie_.exact_match( BOS );
    EOS_ID = word_trie_.exact_match( EOS );
    
    return 0;
}


int Encoder::save( FILE* os ) const {

    if ( word_trie_.save( os ) == -1 ) return -1;
    if ( prefix_trie_.save( os ) == -1 ) return -1;
    if ( suffix_trie_.save( os ) == -1 ) return -1;
    
    return 0;
}


int Encoder::build( const std::vector<std::string>& word, const std::vector<std::string>& prefix, const std::vector<std::string>& suffix ) {
    
    if ( word_trie_.build( word ) == -1 ) return -1;
    if ( prefix_trie_.build( prefix ) == -1 ) return -1;
    if ( suffix_trie_.build_reverse( suffix ) == -1 ) return -1;
    return 0;
}


size_t Encoder::word_num() const {

    return word_trie_.key_num();
}


size_t Encoder::prefix_num() const {

    return prefix_trie_.key_num();
}


size_t Encoder::suffix_num() const {

    return suffix_trie_.key_num();
}
    
    
}

#endif
