#ifndef __STRINGVECTOR__
#define __STRINGVECTOR__

#include <cstring>
#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>

#include "util.h"

namespace SD {

class StringVector {
  private:
    std::vector<std::string> str_;
  public:
    StringVector() {};
    StringVector( const StringVector& obj );
    ~StringVector() {};
    StringVector& operator=( const StringVector& obj );
    int read( FILE* is );
    int load( FILE* is );
    int save( FILE* os ) const;
    void build( const std::vector<std::string>& str );
    const char* operator[]( uint i ) const;
    const std::vector<std::string>& vector() const;
    uint size() const;
};
    

int StringVector::read( FILE* is ) {

    uint size = 0;
    uint i = 0;
    char* s = new char[BUFF_SIZE];
    while ( fgets( s, BUFF_SIZE, is ) != NULL ) {
	if ( s[strlen(s)-1] != '\n' ) {
	    fprintf( stderr, HERE "too long input\n" );
	    return -1;
	}
	s[strlen(s)-1] = '\0';

	if ( i == 0 ) {
	    size = atoi( s );
	    str_.reserve( size );
	}else {
	    str_.push_back( std::string(s) );
	}
	++i;
	if ( i == size + 1 ) break;
    }
    delete[] s;

    return 0;
}


int StringVector::load( FILE* is ) {

    uint size;
    if ( fread( &size, sizeof( uint ), 1, is ) != 1 ) {
	fprintf( stderr, HERE "failed to load\n" );
	return -1;
    }
    str_.resize( size );
    for ( uint i = 0; i < size; ++i ) {
	uint size;
	if ( fread( &size, sizeof( uint ), 1, is ) != 1 ) {
	    fprintf( stderr, HERE "failed to load\n" );
	    return -1;
	}
	char buff[size];
	if ( fread( buff, sizeof( char ), size, is ) != size ) {
	    fprintf( stderr, HERE "failed to load\n" );
	    return -1;
	}
	str_[i].assign( buff );
    }
    
    return 0;
}


int StringVector::save( FILE* os ) const {

    uint size = str_.size();
    if ( fwrite( &size, sizeof( uint ), 1, os ) != 1 ) {
	fprintf( stderr, HERE "failed to save\n" );
	return -1;
    }
    for ( uint i = 0; i < size; ++i ) {
	uint size2 = str_[i].size() + 1; // including null character
	if ( fwrite( &size2, sizeof( uint ), 1, os ) != 1 ) {
	    fprintf( stderr, HERE "failed to save\n" );
	    return -1;
	}
	if ( fwrite( str_[i].c_str(), sizeof( char ), size2, os ) != size2 ) {
	    fprintf( stderr, HERE "failed to save\n" );
	    return -1;
	}
    }
    
    return 0;
}


const std::vector<std::string>& StringVector::vector() const {

    return str_;
}


const char* StringVector::operator[]( uint i ) const {

    return str_[i].c_str();
}


uint StringVector::size() const {

    return str_.size();
}


}

#endif

