#include "timer.h"

const unsigned long long Timer::overhead = Timer::getOverhead ();
const long double Timer::clock = Timer::getCPUClock ();

long double Timer::getCPUClock () {

#ifdef USE_REAL_CLOCK
  register unsigned long long start, end;
  timeval _s_t, _e_t;
  register int min_interval = 50000;
  long interval = 0;
  gettimeofday (&_s_t, NULL);
  start = rdtsc ();
  gettimeofday (&_e_t, NULL);
  while ((_e_t.tv_sec - _s_t.tv_sec) * 1000000 +
         _e_t.tv_usec - _s_t.tv_usec < min_interval)
    gettimeofday (&_e_t, NULL);
  
  end = rdtsc ();
  interval = (_e_t.tv_sec - _s_t.tv_sec) * 1000000 +
             _e_t.tv_usec - _s_t.tv_usec;
  return (end - start - overhead) / interval;
#else
  return 3200.0;
#endif
}

void Timer::printElapsed () const {

    if ( 1 <= _trial ) {
	if ( (_elapsed/clock)/1000 < 1000 ) {
	    fprintf ( stderr, "%-10s : %8.4Lf ms.", _label, (_elapsed/clock)/1000 );
	}else {
	    fprintf ( stderr, "%-10s : %8.4Lf sec.", _label, (_elapsed/clock)/(1000*1000) );
	}
	/*
	else if ( (_elapsed/clock)/(1000*1000) < 60 ) {
	    fprintf ( stderr, "%-10s : %8.4Lf sec.", _label, (_elapsed/clock)/(1000*1000) );
	}else if ( (_elapsed/clock)/(1000*1000*60) < 60 ) {
	    fprintf ( stderr, "%-10s : %8.4Lf min.", _label, (_elapsed/clock)/(1000*1000*60) );
	}else {
	    fprintf ( stderr, "%-10s : %8.4Lf hour", _label, (_elapsed/clock)/(1000*1000*60*60) );
	}
	*/
    }

    if ( _trial == 1 ) {
	fprintf ( stderr, "\n" );
    }else {
	fprintf ( stderr, " (%.4Lf %s/sec.)\n", _trial/((_elapsed/clock)/1000/1000), _unit );
    }
};

